# coding: utf-8
import typing

from django.conf import settings
from itsdangerous import (TimedJSONWebSignatureSerializer, JSONWebSignatureSerializer)
from itsdangerous.exc import (BadSignature, BadTimeSignature)

from .exceptions import (JwtExpireError, JwtNotRightError, )
from .utils import (get_redis_cache, get_header_token, remove_jwt_token, )
from auth_micro import settings as jwt_settings


def jwt_token(
        user_data: typing.Any,
        secret_key: str=settings.JWT_AUTH_SECRET_KEY,
) -> typing.Any:
    """
     加密用户数据
    :param user_data: 加密的数据
    :param secret_key: 加密的key
    :return:
    """
    serializer = JSONWebSignatureSerializer(secret_key)
    return serializer.dumps(user_data)


def jwt_token_expire(
        user_data: typing.Any,
        secret_key: str=settings.JWT_AUTH_SECRET_KEY,
        expire_time: int=None,
) -> typing.Any:
    """

    :param user_data:
    :param secret_key:
    :param expire_time:
    :return:
    """
    expire_time = expire_time or getattr(
        settings, 'JWT_EXPIRE_TIME', jwt_settings.JWT_EXPIRE_TIME)
    serializer = TimedJSONWebSignatureSerializer(
        secret_key, expires_in=expire_time)
    return serializer.dumps(user_data)


def decode_jwt_token(
        request: typing.Any,
        secret_key: str=settings.JWT_AUTH_SECRET_KEY,
        cache_name: str='default',
        token_key: str='',
) -> typing.Any:
    """
    解密token中的用户数据
    :param request: Request实例
    :param secret_key:
    :param cache_name: cache名称
    :param token_key: jwt token
    :return:
    """
    token_key = token_key or get_header_token(request)
    serializer = JSONWebSignatureSerializer(secret_key)
    try:
        user_data = serializer.loads(token_key)
    except (BadSignature, Exception) as e:
        raise JwtNotRightError
    else:
        user_pk = getattr(settings, 'JWT_USER_PK', jwt_settings.JWT_USER_PK)
        user_id = user_data.get(user_pk, '')
        check_jwt_token(user_id, token_key, cache_name)
    request.user = user_data
    return user_data, None


def check_jwt_token(
        key: str,
        token_key: str,
        cache_name: str='default',
) -> typing.Any:
    """
    检查jwt token是否正确
    :param key: redis 的key
    :param token_key: 解密出来的jwt token
    :param cache_name: 缓存的名字
    :return:
    """
    cache_user_token_key = get_redis_cache(key, cache_name)
    if not cache_user_token_key:
        # token过期
        raise JwtExpireError
    if cache_user_token_key != token_key:
        # 不是最新的/正确的token
        raise JwtNotRightError
    return True


def decode_jwt_token_expire(
        request: typing.Any,
        secret_key: str=settings.JWT_AUTH_SECRET_KEY,
        cache_name: str='default',
        expire_time: int=None,
        token_key: str='',
) -> typing.Any:
    """
    解密token中的用户数据
    :param request: Request实例
    :param secret_key:
    :param cache_name: cache名称
    :param expire_time: 过期时间, 为None时表示永远不过期
    :param token_key: jwt token
    :return:
    """
    token_key = token_key or get_header_token(request)
    serializer = TimedJSONWebSignatureSerializer(secret_key, expires_in=expire_time)
    try:
        user_data = serializer.loads(token_key)
    except (BadSignature, BadTimeSignature, Exception):
        raise JwtNotRightError
    else:
        user_pk = getattr(settings, 'JWT_USER_PK', jwt_settings.JWT_USER_PK)
        user_id = user_data.get(user_pk, '')
        check_jwt_token(user_id, token_key, cache_name)
    request.user = user_data
    return user_data, None


def jwt_logout(
        key: str,
        cache_name: str='default',
) -> typing.NoReturn:
    """
    注销用户token
    :param key:
    :param cache_name:
    :return:
    """
    remove_jwt_token(key, cache_name)



# coding: utf-8
import typing

from django.core.cache import caches
from django.conf import settings
from .exceptions import (TokenHeaderError, )
from . import settings as jwt_settings


def get_redis_cache(
        key: str,
        cache_name: str='default',
) -> typing.Any:
    """
    获取redis数据
    :param key:
    :param cache_name:
    :return:
    """
    return caches[cache_name].get(key)


def set_redis_cache(
        key: str,
        data: typing.Any,
        expire_time: int=None,
        cache_name: str='default'
) -> typing.Any:
    """
    将jwt数据储存到redis中
    :param key:
    :param data:
    :param expire_time:
    :param cache_name: cache 名称
    :return:
    """
    assert key, 'The key is required!'
    caches[cache_name].set(key, data, timeout=expire_time)


def remove_jwt_token(
        key: str,
        cache_name: str='default'
) -> typing.NoReturn:
    """
    删除redis中的jwt数据
    :param key:
    :param cache_name:
    :return:
    """
    assert key, 'The key is required!'
    caches[cache_name].delete(key)


def get_header_token(request):
    """
    获取header中的jwt token
    :param request:
    :return:
    """
    token_key = request.META.get('HTTP_AUTHORIZATION', '')
    jwt_token_key = getattr(settings, 'JWT_TOKEN_KEY', jwt_settings.JWT_TOKEN_KEY)
    if token_key is not None:
        return token_key.replace(jwt_token_key, '').replace(' ', '')
    raise TokenHeaderError



# coding: utf-8


class JwtDecodeError(Exception):
    def __str__(self):
        return f'Jwt decode failed!'


class JwtExpireError(Exception):
    def __str__(self):
        return f'Jwt token expired!'


class JwtNotRightError(Exception):
    def __str__(self):
        return f'Jwt token not right!'


class TokenHeaderError(Exception):
    def __str__(self):
        return f'The header not have token!'






# coding: utf-8
from rest_framework.request import Request

from .auth import decode_jwt_token


def jwt_login(func):
    def wrapper(*args, **kwargs):
        request = None
        for count, thing in enumerate(args):
            if type(thing) is Request:
                request = args[count]
                break
        decode_jwt_token(request)
        return func(*args, **kwargs)
    return wrapper



# coding: utf-8
from setuptools import setup, find_packages

setup(
    name='microjwtauth',
    version='0.1.1',
    description='Django microservice JWT authentication',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    author='ivan',
    url='https://github.com/goupper/jwt-auth-micro',
    author_email='chongwuwy@163.com',
    license='MIT',
    packages=find_packages(),
    include_package_data=False,
    zip_safe=True,
)
